import type { ActionManifest } from '../../build/webpack/plugins/flight-client-entry-plugin';
import type { NextFontManifest } from '../../build/webpack/plugins/next-font-manifest-plugin';
import type { LoadableManifest } from '../load-components';
import type { MiddlewareManifest } from '../../build/webpack/plugins/middleware-plugin';
import type { PagesManifest } from '../../build/webpack/plugins/pages-manifest-plugin';
import type { AppBuildManifest } from '../../build/webpack/plugins/app-build-manifest-plugin';
import type { BuildManifest } from '../get-page-files';
import type { NextConfigComplete } from '../config-shared';
import { APP_BUILD_MANIFEST, APP_PATHS_MANIFEST, BUILD_MANIFEST, MIDDLEWARE_MANIFEST, NEXT_FONT_MANIFEST, PAGES_MANIFEST, SERVER_REFERENCE_MANIFEST, REACT_LOADABLE_MANIFEST } from '../../shared/lib/constants';
import type { SetupOpts } from '../lib/router-utils/setup-dev-bundler';
import type { Route } from '../../build/swc';
export interface InstrumentationDefinition {
    files: string[];
    name: 'instrumentation';
}
export type TurbopackMiddlewareManifest = MiddlewareManifest & {
    instrumentation?: InstrumentationDefinition;
};
export declare function mergeBuildManifests(manifests: Iterable<BuildManifest>): Partial<BuildManifest> & Pick<BuildManifest, "pages">;
export declare function mergeAppBuildManifests(manifests: Iterable<AppBuildManifest>): AppBuildManifest;
export declare function mergePagesManifests(manifests: Iterable<PagesManifest>): PagesManifest;
export declare function mergeMiddlewareManifests(manifests: Iterable<TurbopackMiddlewareManifest>): MiddlewareManifest;
export declare function mergeActionManifests(manifests: Iterable<ActionManifest>): Promise<ActionManifest>;
export declare function mergeFontManifests(manifests: Iterable<NextFontManifest>): NextFontManifest;
export declare function mergeLoadableManifests(manifests: Iterable<LoadableManifest>): LoadableManifest;
export declare function getTurbopackJsConfig(dir: string, nextConfig: NextConfigComplete): Promise<{
    compilerOptions: Record<string, any>;
}>;
export declare function readPartialManifest<T>(distDir: string, name: typeof MIDDLEWARE_MANIFEST | typeof BUILD_MANIFEST | typeof APP_BUILD_MANIFEST | typeof PAGES_MANIFEST | typeof APP_PATHS_MANIFEST | `${typeof SERVER_REFERENCE_MANIFEST}.json` | `${typeof NEXT_FONT_MANIFEST}.json` | typeof REACT_LOADABLE_MANIFEST, pageName: string, type?: 'pages' | 'app' | 'app-route' | 'middleware' | 'instrumentation'): Promise<T>;
export type BuildManifests = Map<string, BuildManifest>;
export type AppBuildManifests = Map<string, AppBuildManifest>;
export type PagesManifests = Map<string, PagesManifest>;
export type AppPathsManifests = Map<string, PagesManifest>;
export type MiddlewareManifests = Map<string, TurbopackMiddlewareManifest>;
export type ActionManifests = Map<string, ActionManifest>;
export type FontManifests = Map<string, NextFontManifest>;
export type LoadableManifests = Map<string, LoadableManifest>;
export type CurrentEntrypoints = Map<string, Route>;
export declare function loadMiddlewareManifest(distDir: string, middlewareManifests: MiddlewareManifests, pageName: string, type: 'pages' | 'app' | 'app-route' | 'middleware' | 'instrumentation'): Promise<void>;
export declare function loadBuildManifest(distDir: string, buildManifests: BuildManifests, pageName: string, type?: 'app' | 'pages'): Promise<void>;
export declare function loadAppBuildManifest(distDir: string, appBuildManifests: AppBuildManifests, pageName: string): Promise<void>;
export declare function loadPagesManifest(distDir: string, pagesManifests: PagesManifests, pageName: string): Promise<void>;
export declare function loadAppPathManifest(distDir: string, appPathsManifests: AppPathsManifests, pageName: string, type?: 'app' | 'app-route'): Promise<void>;
export declare function loadActionManifest(distDir: string, actionManifests: ActionManifests, pageName: string): Promise<void>;
export declare function loadFontManifest(distDir: string, fontManifests: FontManifests, pageName: string, type?: 'app' | 'pages'): Promise<void>;
export declare function loadLoadableManifest(distDir: string, loadableManifests: LoadableManifests, pageName: string, type?: 'app' | 'pages'): Promise<void>;
export declare function writeManifests(opts: SetupOpts, distDir: string, buildManifests: BuildManifests, appBuildManifests: AppBuildManifests, pagesManifests: PagesManifests, appPathsManifests: AppPathsManifests, middlewareManifests: MiddlewareManifests, actionManifests: ActionManifests, fontManifests: FontManifests, loadableManifests: LoadableManifests, currentEntrypoints: CurrentEntrypoints): Promise<void>;
