"use strict";
Object.defineProperty(exports, "__esModule", {
    value: true
});
0 && (module.exports = {
    mergeBuildManifests: null,
    mergeAppBuildManifests: null,
    mergePagesManifests: null,
    mergeMiddlewareManifests: null,
    mergeActionManifests: null,
    mergeFontManifests: null,
    mergeLoadableManifests: null,
    getTurbopackJsConfig: null,
    readPartialManifest: null,
    loadMiddlewareManifest: null,
    loadBuildManifest: null,
    loadAppBuildManifest: null,
    loadPagesManifest: null,
    loadAppPathManifest: null,
    loadActionManifest: null,
    loadFontManifest: null,
    loadLoadableManifest: null,
    writeManifests: null
});
function _export(target, all) {
    for(var name in all)Object.defineProperty(target, name, {
        enumerable: true,
        get: all[name]
    });
}
_export(exports, {
    mergeBuildManifests: function() {
        return mergeBuildManifests;
    },
    mergeAppBuildManifests: function() {
        return mergeAppBuildManifests;
    },
    mergePagesManifests: function() {
        return mergePagesManifests;
    },
    mergeMiddlewareManifests: function() {
        return mergeMiddlewareManifests;
    },
    mergeActionManifests: function() {
        return mergeActionManifests;
    },
    mergeFontManifests: function() {
        return mergeFontManifests;
    },
    mergeLoadableManifests: function() {
        return mergeLoadableManifests;
    },
    getTurbopackJsConfig: function() {
        return getTurbopackJsConfig;
    },
    readPartialManifest: function() {
        return readPartialManifest;
    },
    loadMiddlewareManifest: function() {
        return loadMiddlewareManifest;
    },
    loadBuildManifest: function() {
        return loadBuildManifest;
    },
    loadAppBuildManifest: function() {
        return loadAppBuildManifest;
    },
    loadPagesManifest: function() {
        return loadPagesManifest;
    },
    loadAppPathManifest: function() {
        return loadAppPathManifest;
    },
    loadActionManifest: function() {
        return loadActionManifest;
    },
    loadFontManifest: function() {
        return loadFontManifest;
    },
    loadLoadableManifest: function() {
        return loadLoadableManifest;
    },
    writeManifests: function() {
        return writeManifests;
    }
});
const _pathtoregexp = require("next/dist/compiled/path-to-regexp");
const _actionencryptionutils = require("../app-render/action-encryption-utils");
const _loadjsconfig = /*#__PURE__*/ _interop_require_default(require("../../build/load-jsconfig"));
const _path = require("path");
const _promises = require("fs/promises");
const _constants = require("../../shared/lib/constants");
const _writeatomic = require("../../lib/fs/write-atomic");
const _nextjsrequirecachehotreloader = require("../../build/webpack/plugins/nextjs-require-cache-hot-reloader");
const _buildmanifestplugin = require("../../build/webpack/plugins/build-manifest-plugin");
const _generateinterceptionroutesrewrites = require("../../lib/generate-interception-routes-rewrites");
function _interop_require_default(obj) {
    return obj && obj.__esModule ? obj : {
        default: obj
    };
}
function mergeBuildManifests(manifests) {
    const manifest = {
        pages: {
            "/_app": []
        },
        // Something in next.js depends on these to exist even for app dir rendering
        devFiles: [],
        ampDevFiles: [],
        polyfillFiles: [],
        lowPriorityFiles: [
            "static/development/_ssgManifest.js",
            "static/development/_buildManifest.js"
        ],
        rootMainFiles: [],
        ampFirstPages: []
    };
    for (const m of manifests){
        Object.assign(manifest.pages, m.pages);
        if (m.rootMainFiles.length) manifest.rootMainFiles = m.rootMainFiles;
    }
    return manifest;
}
function mergeAppBuildManifests(manifests) {
    const manifest = {
        pages: {}
    };
    for (const m of manifests){
        Object.assign(manifest.pages, m.pages);
    }
    return manifest;
}
function mergePagesManifests(manifests) {
    const manifest = {};
    for (const m of manifests){
        Object.assign(manifest, m);
    }
    return manifest;
}
function mergeMiddlewareManifests(manifests) {
    const manifest = {
        version: 2,
        middleware: {},
        sortedMiddleware: [],
        functions: {}
    };
    let instrumentation = undefined;
    for (const m of manifests){
        Object.assign(manifest.functions, m.functions);
        Object.assign(manifest.middleware, m.middleware);
        if (m.instrumentation) {
            instrumentation = m.instrumentation;
        }
    }
    const updateFunctionDefinition = (fun)=>{
        return {
            ...fun,
            files: [
                ...(instrumentation == null ? void 0 : instrumentation.files) ?? [],
                ...fun.files
            ]
        };
    };
    for (const key of Object.keys(manifest.middleware)){
        const value = manifest.middleware[key];
        manifest.middleware[key] = updateFunctionDefinition(value);
    }
    for (const key of Object.keys(manifest.functions)){
        const value = manifest.functions[key];
        manifest.functions[key] = updateFunctionDefinition(value);
    }
    for (const fun of Object.values(manifest.functions).concat(Object.values(manifest.middleware))){
        for (const matcher of fun.matchers){
            if (!matcher.regexp) {
                matcher.regexp = (0, _pathtoregexp.pathToRegexp)(matcher.originalSource, [], {
                    delimiter: "/",
                    sensitive: false,
                    strict: true
                }).source.replaceAll("\\/", "/");
            }
        }
    }
    manifest.sortedMiddleware = Object.keys(manifest.middleware);
    return manifest;
}
async function mergeActionManifests(manifests) {
    const manifest = {
        node: {},
        edge: {},
        encryptionKey: await (0, _actionencryptionutils.generateRandomActionKeyRaw)(true)
    };
    function mergeActionIds(actionEntries, other) {
        for(const key in other){
            const action = actionEntries[key] ??= {
                workers: {},
                layer: {}
            };
            Object.assign(action.workers, other[key].workers);
            Object.assign(action.layer, other[key].layer);
        }
    }
    for (const m of manifests){
        mergeActionIds(manifest.node, m.node);
        mergeActionIds(manifest.edge, m.edge);
    }
    return manifest;
}
function mergeFontManifests(manifests) {
    const manifest = {
        app: {},
        appUsingSizeAdjust: false,
        pages: {},
        pagesUsingSizeAdjust: false
    };
    for (const m of manifests){
        Object.assign(manifest.app, m.app);
        Object.assign(manifest.pages, m.pages);
        manifest.appUsingSizeAdjust = manifest.appUsingSizeAdjust || m.appUsingSizeAdjust;
        manifest.pagesUsingSizeAdjust = manifest.pagesUsingSizeAdjust || m.pagesUsingSizeAdjust;
    }
    return manifest;
}
function mergeLoadableManifests(manifests) {
    const manifest = {};
    for (const m of manifests){
        Object.assign(manifest, m);
    }
    return manifest;
}
async function getTurbopackJsConfig(dir, nextConfig) {
    const { jsConfig } = await (0, _loadjsconfig.default)(dir, nextConfig);
    return jsConfig ?? {
        compilerOptions: {}
    };
}
async function readPartialManifest(distDir, name, pageName, type = "pages") {
    const manifestPath = _path.posix.join(distDir, `server`, type === "app-route" ? "app" : type, type === "middleware" || type === "instrumentation" ? "" : pageName === "/" ? "index" : pageName === "/index" || pageName.startsWith("/index/") ? `/index${pageName}` : pageName, type === "app" ? "page" : type === "app-route" ? "route" : "", name);
    return JSON.parse(await (0, _promises.readFile)(_path.posix.join(manifestPath), "utf-8"));
}
async function loadMiddlewareManifest(distDir, middlewareManifests, pageName, type) {
    middlewareManifests.set(pageName, await readPartialManifest(distDir, _constants.MIDDLEWARE_MANIFEST, pageName, type));
}
async function loadBuildManifest(distDir, buildManifests, pageName, type = "pages") {
    buildManifests.set(pageName, await readPartialManifest(distDir, _constants.BUILD_MANIFEST, pageName, type));
}
async function loadAppBuildManifest(distDir, appBuildManifests, pageName) {
    appBuildManifests.set(pageName, await readPartialManifest(distDir, _constants.APP_BUILD_MANIFEST, pageName, "app"));
}
async function loadPagesManifest(distDir, pagesManifests, pageName) {
    pagesManifests.set(pageName, await readPartialManifest(distDir, _constants.PAGES_MANIFEST, pageName));
}
async function loadAppPathManifest(distDir, appPathsManifests, pageName, type = "app") {
    appPathsManifests.set(pageName, await readPartialManifest(distDir, _constants.APP_PATHS_MANIFEST, pageName, type));
}
async function loadActionManifest(distDir, actionManifests, pageName) {
    actionManifests.set(pageName, await readPartialManifest(distDir, `${_constants.SERVER_REFERENCE_MANIFEST}.json`, pageName, "app"));
}
async function loadFontManifest(distDir, fontManifests, pageName, type = "pages") {
    fontManifests.set(pageName, await readPartialManifest(distDir, `${_constants.NEXT_FONT_MANIFEST}.json`, pageName, type));
}
async function loadLoadableManifest(distDir, loadableManifests, pageName, type = "pages") {
    loadableManifests.set(pageName, await readPartialManifest(distDir, _constants.REACT_LOADABLE_MANIFEST, pageName, type));
}
async function writeBuildManifest(distDir, buildManifests, currentEntrypoints, rewrites) {
    const buildManifest = mergeBuildManifests(buildManifests.values());
    const buildManifestPath = (0, _path.join)(distDir, _constants.BUILD_MANIFEST);
    const middlewareBuildManifestPath = (0, _path.join)(distDir, "server", `${_constants.MIDDLEWARE_BUILD_MANIFEST}.js`);
    const interceptionRewriteManifestPath = (0, _path.join)(distDir, "server", `${_constants.INTERCEPTION_ROUTE_REWRITE_MANIFEST}.js`);
    (0, _nextjsrequirecachehotreloader.deleteCache)(buildManifestPath);
    (0, _nextjsrequirecachehotreloader.deleteCache)(middlewareBuildManifestPath);
    (0, _nextjsrequirecachehotreloader.deleteCache)(interceptionRewriteManifestPath);
    await (0, _writeatomic.writeFileAtomic)(buildManifestPath, JSON.stringify(buildManifest, null, 2));
    await (0, _writeatomic.writeFileAtomic)(middlewareBuildManifestPath, `self.__BUILD_MANIFEST=${JSON.stringify(buildManifest)};`);
    const interceptionRewrites = JSON.stringify(rewrites.beforeFiles.filter(_generateinterceptionroutesrewrites.isInterceptionRouteRewrite));
    await (0, _writeatomic.writeFileAtomic)(interceptionRewriteManifestPath, `self.__INTERCEPTION_ROUTE_REWRITE_MANIFEST=${JSON.stringify(interceptionRewrites)};`);
    const content = {
        __rewrites: rewrites ? (0, _buildmanifestplugin.normalizeRewritesForBuildManifest)(rewrites) : {
            afterFiles: [],
            beforeFiles: [],
            fallback: []
        },
        ...Object.fromEntries([
            ...currentEntrypoints.keys()
        ].map((pathname)=>[
                pathname,
                `static/chunks/pages${pathname === "/" ? "/index" : pathname}.js`
            ])),
        sortedPages: [
            ...currentEntrypoints.keys()
        ]
    };
    const buildManifestJs = `self.__BUILD_MANIFEST = ${JSON.stringify(content)};self.__BUILD_MANIFEST_CB && self.__BUILD_MANIFEST_CB()`;
    await (0, _writeatomic.writeFileAtomic)((0, _path.join)(distDir, "static", "development", "_buildManifest.js"), buildManifestJs);
    await (0, _writeatomic.writeFileAtomic)((0, _path.join)(distDir, "static", "development", "_ssgManifest.js"), _buildmanifestplugin.srcEmptySsgManifest);
}
async function writeFallbackBuildManifest(distDir, buildManifests) {
    const fallbackBuildManifest = mergeBuildManifests([
        buildManifests.get("_app"),
        buildManifests.get("_error")
    ].filter(Boolean));
    const fallbackBuildManifestPath = (0, _path.join)(distDir, `fallback-${_constants.BUILD_MANIFEST}`);
    (0, _nextjsrequirecachehotreloader.deleteCache)(fallbackBuildManifestPath);
    await (0, _writeatomic.writeFileAtomic)(fallbackBuildManifestPath, JSON.stringify(fallbackBuildManifest, null, 2));
}
async function writeAppBuildManifest(distDir, appBuildManifests) {
    const appBuildManifest = mergeAppBuildManifests(appBuildManifests.values());
    const appBuildManifestPath = (0, _path.join)(distDir, _constants.APP_BUILD_MANIFEST);
    (0, _nextjsrequirecachehotreloader.deleteCache)(appBuildManifestPath);
    await (0, _writeatomic.writeFileAtomic)(appBuildManifestPath, JSON.stringify(appBuildManifest, null, 2));
}
async function writePagesManifest(distDir, pagesManifests) {
    const pagesManifest = mergePagesManifests(pagesManifests.values());
    const pagesManifestPath = (0, _path.join)(distDir, "server", _constants.PAGES_MANIFEST);
    (0, _nextjsrequirecachehotreloader.deleteCache)(pagesManifestPath);
    await (0, _writeatomic.writeFileAtomic)(pagesManifestPath, JSON.stringify(pagesManifest, null, 2));
}
async function writeAppPathsManifest(distDir, appPathsManifests) {
    const appPathsManifest = mergePagesManifests(appPathsManifests.values());
    const appPathsManifestPath = (0, _path.join)(distDir, "server", _constants.APP_PATHS_MANIFEST);
    (0, _nextjsrequirecachehotreloader.deleteCache)(appPathsManifestPath);
    await (0, _writeatomic.writeFileAtomic)(appPathsManifestPath, JSON.stringify(appPathsManifest, null, 2));
}
async function writeMiddlewareManifest(distDir, middlewareManifests) {
    const middlewareManifest = mergeMiddlewareManifests(middlewareManifests.values());
    const middlewareManifestPath = (0, _path.join)(distDir, "server", _constants.MIDDLEWARE_MANIFEST);
    (0, _nextjsrequirecachehotreloader.deleteCache)(middlewareManifestPath);
    await (0, _writeatomic.writeFileAtomic)(middlewareManifestPath, JSON.stringify(middlewareManifest, null, 2));
}
async function writeActionManifest(distDir, actionManifests) {
    const actionManifest = await mergeActionManifests(actionManifests.values());
    const actionManifestJsonPath = (0, _path.join)(distDir, "server", `${_constants.SERVER_REFERENCE_MANIFEST}.json`);
    const actionManifestJsPath = (0, _path.join)(distDir, "server", `${_constants.SERVER_REFERENCE_MANIFEST}.js`);
    const json = JSON.stringify(actionManifest, null, 2);
    (0, _nextjsrequirecachehotreloader.deleteCache)(actionManifestJsonPath);
    (0, _nextjsrequirecachehotreloader.deleteCache)(actionManifestJsPath);
    await (0, _promises.writeFile)(actionManifestJsonPath, json, "utf-8");
    await (0, _promises.writeFile)(actionManifestJsPath, `self.__RSC_SERVER_MANIFEST=${JSON.stringify(json)}`, "utf-8");
}
async function writeFontManifest(distDir, fontManifests) {
    const fontManifest = mergeFontManifests(fontManifests.values());
    const json = JSON.stringify(fontManifest, null, 2);
    const fontManifestJsonPath = (0, _path.join)(distDir, "server", `${_constants.NEXT_FONT_MANIFEST}.json`);
    const fontManifestJsPath = (0, _path.join)(distDir, "server", `${_constants.NEXT_FONT_MANIFEST}.js`);
    (0, _nextjsrequirecachehotreloader.deleteCache)(fontManifestJsonPath);
    (0, _nextjsrequirecachehotreloader.deleteCache)(fontManifestJsPath);
    await (0, _writeatomic.writeFileAtomic)(fontManifestJsonPath, json);
    await (0, _writeatomic.writeFileAtomic)(fontManifestJsPath, `self.__NEXT_FONT_MANIFEST=${JSON.stringify(json)}`);
}
async function writeLoadableManifest(distDir, loadableManifests) {
    const loadableManifest = mergeLoadableManifests(loadableManifests.values());
    const loadableManifestPath = (0, _path.join)(distDir, _constants.REACT_LOADABLE_MANIFEST);
    const middlewareloadableManifestPath = (0, _path.join)(distDir, "server", `${_constants.MIDDLEWARE_REACT_LOADABLE_MANIFEST}.js`);
    const json = JSON.stringify(loadableManifest, null, 2);
    (0, _nextjsrequirecachehotreloader.deleteCache)(loadableManifestPath);
    (0, _nextjsrequirecachehotreloader.deleteCache)(middlewareloadableManifestPath);
    await (0, _writeatomic.writeFileAtomic)(loadableManifestPath, json);
    await (0, _writeatomic.writeFileAtomic)(middlewareloadableManifestPath, `self.__REACT_LOADABLE_MANIFEST=${JSON.stringify(json)}`);
}
async function writeManifests(opts, distDir, buildManifests, appBuildManifests, pagesManifests, appPathsManifests, middlewareManifests, actionManifests, fontManifests, loadableManifests, currentEntrypoints) {
    await writeBuildManifest(distDir, buildManifests, currentEntrypoints, opts.fsChecker.rewrites);
    await writeAppBuildManifest(distDir, appBuildManifests);
    await writePagesManifest(distDir, pagesManifests);
    await writeAppPathsManifest(distDir, appPathsManifests);
    await writeMiddlewareManifest(distDir, middlewareManifests);
    await writeActionManifest(distDir, actionManifests);
    await writeFontManifest(distDir, fontManifests);
    await writeLoadableManifest(distDir, loadableManifests);
    await writeFallbackBuildManifest(distDir, buildManifests);
}

//# sourceMappingURL=turbopack-utils.js.map